/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.data;

import android.content.Context;
import android.database.Cursor;
import android.provider.ContactsContract.CommonDataKinds.Phone;
import android.provider.ContactsContract.Data;
import android.telephony.PhoneNumberUtils;
import android.util.Log;

import java.util.HashMap;

public class Contact {
    // 联系人缓存，用于存储已查询过的电话号码和对应的联系人姓名
    private static HashMap<String, String> sContactCache;
    // 日志标签
    private static final String TAG = "Contact";

    // 查询联系人的SQL选择语句模板
    // 这个语句用于查询与给定电话号码匹配的联系人姓名
    // 使用PHONE_NUMBERS_EQUAL函数比较电话号码，并限制只查询电话类型的联系人数据
    private static final String CALLER_ID_SELECTION = "PHONE_NUMBERS_EQUAL(" + Phone.NUMBER
    + ",?) AND " + Data.MIMETYPE + "='" + Phone.CONTENT_ITEM_TYPE + "'"
    + " AND " + Data.RAW_CONTACT_ID + " IN "
            + "(SELECT raw_contact_id "
            + " FROM phone_lookup"
            + " WHERE min_match = '+')";

    /**
     * 根据电话号码获取联系人姓名
     * @param context Android上下文对象
     * @param phoneNumber 要查询的电话号码
     * @return 联系人姓名，如果找不到则返回null
     */
    public static String getContact(Context context, String phoneNumber) {
        // 如果缓存不存在，初始化缓存
        if(sContactCache == null) {
            sContactCache = new HashMap<String, String>();
        }

        // 首先检查缓存中是否已有该号码的查询结果
        if(sContactCache.containsKey(phoneNumber)) {
            return sContactCache.get(phoneNumber);
        }

        // 准备查询语句
        // 替换最小匹配长度参数
        String selection = CALLER_ID_SELECTION.replace("+",
                PhoneNumberUtils.toCallerIDMinMatch(phoneNumber));
        
        // 查询联系人数据库
        Cursor cursor = context.getContentResolver().query(
                Data.CONTENT_URI,                   // 数据URI
                new String [] { Phone.DISPLAY_NAME }, // 只查询显示名字段
                selection,                          // 查询条件
                new String[] { phoneNumber },        // 查询参数
                null);                              // 排序方式

        // 处理查询结果
        if (cursor != null && cursor.moveToFirst()) {
            try {
                // 获取联系人姓名
                String name = cursor.getString(0);
                // 将结果存入缓存
                sContactCache.put(phoneNumber, name);
                return name;
            } catch (IndexOutOfBoundsException e) {
                // 处理可能的异常
                Log.e(TAG, " Cursor get string error " + e.toString());
                return null;
            } finally {
                // 确保关闭Cursor
                cursor.close();
            }
        } else {
            // 没有找到匹配的联系人
            Log.d(TAG, "No contact matched with number:" + phoneNumber);
            return null;
        }
    }
}