/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.gtask.remote;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.os.AsyncTask;

import net.micode.notes.R;
import net.micode.notes.ui.NotesListActivity;
import net.micode.notes.ui.NotesPreferenceActivity;


// 用于Google任务同步的异步任务类
public class GTaskASyncTask extends AsyncTask<Void, String, Integer> {

    // 同步通知的固定ID
    private static int GTASK_SYNC_NOTIFICATION_ID = 5234235;

    // 同步完成监听接口
    public interface OnCompleteListener {
        void onComplete();
    }

    private Context mContext;  // 上下文对象
    private NotificationManager mNotifiManager;  // 通知管理器
    private GTaskManager mTaskManager;  // Google任务管理器
    private OnCompleteListener mOnCompleteListener;  // 同步完成监听器

    // 构造函数
    public GTaskASyncTask(Context context, OnCompleteListener listener) {
        mContext = context;
        mOnCompleteListener = listener;
        // 初始化通知管理器
        mNotifiManager = (NotificationManager) mContext
                .getSystemService(Context.NOTIFICATION_SERVICE);
        // 获取Google任务管理器实例
        mTaskManager = GTaskManager.getInstance();
    }

    // 取消同步
    public void cancelSync() {
        mTaskManager.cancelSync();
    }

    // 发布进度更新
    public void publishProgess(String message) {
        publishProgress(new String[] {
            message
        });
    }

    // 显示通知
    private void showNotification(int tickerId, String content) {
        // 创建通知对象
        Notification notification = new Notification(R.drawable.notification, mContext
                .getString(tickerId), System.currentTimeMillis());
        // 设置默认灯光效果
        notification.defaults = Notification.DEFAULT_LIGHTS;
        // 设置自动取消标志
        notification.flags = Notification.FLAG_AUTO_CANCEL;
        
        PendingIntent pendingIntent;
        // 根据通知类型设置不同的跳转页面
        if (tickerId != R.string.ticker_success) {
            // 失败时跳转到设置页面
            pendingIntent = PendingIntent.getActivity(mContext, 0, new Intent(mContext,
                    NotesPreferenceActivity.class), 0);
        } else {
            // 成功时跳转到笔记列表页面
            pendingIntent = PendingIntent.getActivity(mContext, 0, new Intent(mContext,
                    NotesListActivity.class), 0);
        }
        // 设置通知详细信息
        notification.setLatestEventInfo(mContext, mContext.getString(R.string.app_name), content,
                pendingIntent);
        // 显示通知
        mNotifiManager.notify(GTASK_SYNC_NOTIFICATION_ID, notification);
    }

    // 后台执行同步任务
    @Override
    protected Integer doInBackground(Void... unused) {
        // 发布登录进度信息
        publishProgess(mContext.getString(R.string.sync_progress_login, NotesPreferenceActivity
                .getSyncAccountName(mContext)));
        // 执行同步操作
        return mTaskManager.sync(mContext, this);
    }

    // 进度更新处理
    @Override
    protected void onProgressUpdate(String... progress) {
        // 显示同步中通知
        showNotification(R.string.ticker_syncing, progress[0]);
        // 如果是服务上下文，发送广播
        if (mContext instanceof GTaskSyncService) {
            ((GTaskSyncService) mContext).sendBroadcast(progress[0]);
        }
    }

    // 任务执行完成处理
    @Override
    protected void onPostExecute(Integer result) {
        // 根据同步结果显示不同通知
        if (result == GTaskManager.STATE_SUCCESS) {
            // 成功通知
            showNotification(R.string.ticker_success, mContext.getString(
                    R.string.success_sync_account, mTaskManager.getSyncAccount()));
            // 更新最后同步时间
            NotesPreferenceActivity.setLastSyncTime(mContext, System.currentTimeMillis());
        } else if (result == GTaskManager.STATE_NETWORK_ERROR) {
            // 网络错误通知
            showNotification(R.string.ticker_fail, mContext.getString(R.string.error_sync_network));
        } else if (result == GTaskManager.STATE_INTERNAL_ERROR) {
            // 内部错误通知
            showNotification(R.string.ticker_fail, mContext.getString(R.string.error_sync_internal));
        } else if (result == GTaskManager.STATE_SYNC_CANCELLED) {
            // 同步取消通知
            showNotification(R.string.ticker_cancel, mContext
                    .getString(R.string.error_sync_cancelled));
        }
        // 如果有完成监听器，在新线程中执行回调
        if (mOnCompleteListener != null) {
            new Thread(new Runnable() {
                public void run() {
                    mOnCompleteListener.onComplete();
                }
            }).start();
        }
    }
}