/*
 * Copyright (c) 2010-2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.micode.notes.gtask.remote;

import android.app.Activity;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.os.IBinder;

// GTask同步服务类，继承自Service
public class GTaskSyncService extends Service {
    // 定义常量字符串，表示同步动作类型
    public final static String ACTION_STRING_NAME = "sync_action_type";

    // 定义同步动作常量
    public final static int ACTION_START_SYNC = 0;  // 开始同步
    public final static int ACTION_CANCEL_SYNC = 1; // 取消同步
    public final static int ACTION_INVALID = 2;     // 无效动作

    // 定义广播相关常量
    public final static String GTASK_SERVICE_BROADCAST_NAME = "net.micode.notes.gtask.remote.gtask_sync_service";
    public final static String GTASK_SERVICE_BROADCAST_IS_SYNCING = "isSyncing";
    public final static String GTASK_SERVICE_BROADCAST_PROGRESS_MSG = "progressMsg";

    // 静态变量，保存同步任务和同步进度
    private static GTaskASyncTask mSyncTask = null;
    private static String mSyncProgress = "";

    // 开始同步方法
    private void startSync() {
        if (mSyncTask == null) {
            mSyncTask = new GTaskASyncTask(this, new GTaskASyncTask.OnCompleteListener() {
                public void onComplete() {
                    mSyncTask = null;
                    sendBroadcast("");
                    stopSelf(); // 任务完成后停止服务
                }
            });
            sendBroadcast("");
            mSyncTask.execute(); // 执行同步任务
        }
    }

    // 取消同步方法
    private void cancelSync() {
        if (mSyncTask != null) {
            mSyncTask.cancelSync(); // 取消同步任务
        }
    }

    @Override
    public void onCreate() {
        mSyncTask = null; // 服务创建时初始化同步任务为null
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        Bundle bundle = intent.getExtras();
        if (bundle != null && bundle.containsKey(ACTION_STRING_NAME)) {
            // 根据传入的动作类型执行相应操作
            switch (bundle.getInt(ACTION_STRING_NAME, ACTION_INVALID)) {
                case ACTION_START_SYNC:
                    startSync(); // 开始同步
                    break;
                case ACTION_CANCEL_SYNC:
                    cancelSync(); // 取消同步
                    break;
                default:
                    break;
            }
            return START_STICKY; // 服务被杀死后自动重启
        }
        return super.onStartCommand(intent, flags, startId);
    }

    @Override
    public void onLowMemory() {
        // 内存不足时取消同步
        if (mSyncTask != null) {
            mSyncTask.cancelSync();
        }
    }

    // 绑定服务时调用，返回null表示不支持绑定
    public IBinder onBind(Intent intent) {
        return null;
    }

    // 发送广播方法，通知同步状态和进度
    public void sendBroadcast(String msg) {
        mSyncProgress = msg;
        Intent intent = new Intent(GTASK_SERVICE_BROADCAST_NAME);
        intent.putExtra(GTASK_SERVICE_BROADCAST_IS_SYNCING, mSyncTask != null);
        intent.putExtra(GTASK_SERVICE_BROADCAST_PROGRESS_MSG, msg);
        sendBroadcast(intent);
    }

    // 静态方法，从Activity启动同步
    public static void startSync(Activity activity) {
        GTaskManager.getInstance().setActivityContext(activity);
        Intent intent = new Intent(activity, GTaskSyncService.class);
        intent.putExtra(GTaskSyncService.ACTION_STRING_NAME, GTaskSyncService.ACTION_START_SYNC);
        activity.startService(intent);
    }

    // 静态方法，取消同步
    public static void cancelSync(Context context) {
        Intent intent = new Intent(context, GTaskSyncService.class);
        intent.putExtra(GTaskSyncService.ACTION_STRING_NAME, GTaskSyncService.ACTION_CANCEL_SYNC);
        context.startService(intent);
    }

    // 检查是否正在同步
    public static boolean isSyncing() {
        return mSyncTask != null;
    }

    // 获取同步进度字符串
    public static String getProgressString() {
        return mSyncProgress;
    }
}